﻿var htmlLatexRender = function () {
    //获取宽
    function getWidth(el) {
        return Math.max(el.clientWidth, el.scrollWidth);      //clientWidth:元素可见区域的宽度  clientWidth=padding+width-滚动条;   scrollWidth：元素实际内容的宽度 scrollWidth=padding+width（元素实际的内容）
    }

    //获取高
    function getHeight(el) {
        return Math.max(el.clientHeight, el.scrollHeight);
    }

    //获取属性
    function getAttr(el, attr) {
        return (el.nodeType != 1 ? null : el.getAttribute(attr)); //nodeType=1元素节点;  =2属性节点
    }

    //新增修改样式
    function setStyle(el, attr, val) {
        switch (typeof attr) {
            case "object":
                for (var jn in attr) {
                    el.style[jn] = attr[jn];
                }
                break;
            case "string":
                el.style[attr] = val;
                break;
            default:
        }
    }

    //获取同辈前一个EL
    function prevEL(el, p) {
        if (el == null) { return null; }
        try {
            if (p == true) {
                var nel = el.parentNode;     //父节点
                if (nel.previousElementSibling) {        //previousElementSibling 返回指定元素的前一个兄弟元素
                    return nel.previousElementSibling;
                }

                while (nel = nel.previousSibling) {
                    if (nel.nodeType == 1) { return nel; }
                }
            }
            else {
                if (el.previousElementSibling) {
                    return el.previousElementSibling;
                }

                while (el = el.previousSibling) {
                    if (el.nodeType == 1) { return el; }
                }
            }
        } catch (e) { }
        return null;
    }

    //获取同辈后一个EL
    function nextEL(el, p) {
        if (el == null) { return null; }
        try {
            if (p == true) {
                var nel = el.parentNode;
                if (nel.nextElementSibling) {
                    return nel.nextElementSibling;
                }

                while (nel = nel.nextSibling) {
                    if (nel.nodeType == 1) { return nel; }
                }
            }
            else {
                if (el.nextElementSibling) {
                    return el.nextElementSibling;
                }

                while (el = el.nextSibling) {
                    if (el.nodeType == 1) { return el; }
                }
            }
        } catch (e) { }
        return null;
    }

    //遍历给定元素的所有集合并执行给定的操作
    function scanAction(el, delay) {
        if (el == null || delay == null) { return; }
        var els = el.children || el.childNodes;       //children：返回被选元素的所有直接子元素；  childNodes：返回节点的子节点集合，以 NodeList 对象
        delay(el);                                    //delay：回调参数函数
        for (var i = 0; i < els.length; i++) {
            scanAction(els[i], delay);               //递归调用
        }
    }

    //获取给定元素的所有节点集合
    function descendants(el, rv, delay, d) {         //对应参数：delay回调函数
        //console.log(el);                           //el每个元素节点
        if (el == null) { return; }
        ++d;
        var node;
        var els = el.children || el.childNodes;
        for (var i = 0; i < els.length; i++) {
            node = els[i];
            if (node.nodeType == 1) {
                if (delay(node)) {
                    rv.push({ "n": node, "d": d });
                }
                descendants(node, rv, delay, d);           //递归调用
            }
        }
    }

    //初始布局
    this.LayOut = function (el, clear) {
        var list = el.getElementsByTagName("span");
        for (var i = 0; i < list.length; i++) {
            if (getAttr(list[i], "mathtag") != "math") {          //list[i]结果为：span  mathtag="math"
                continue;
            }

            embLetter(list[i]);//调整字体
            //某些时候用到动态调整字体，这一步是还原之前拉伸的宽高
            if (clear) {
                scanAction(list[i], function (el) {
                    if (el.nodeType != 1) { return; }
                    switch (getAttr(el, "muststretch")) {
                        case "v":
                            el.style.height = "1px";
                            break;
                        case "h":
                            el.style.width = "1px";
                            break;
                    }
                });
            }

            //积分符号初始化调整
            var arrInt = [];
            descendants(list[i], arrInt, function (el) {
                if ($(el).html() == "∫") {
                    return true;
                }

            }, 0);
            for (var j = 0; j < arrInt.length; j++) {
                if (arrInt[j].n.length) { return; }
                dsInt(arrInt[j].n);
            }

            if (getAttr(list[i], "dealflag") == "1") {
                updateLayout(list[i]);       //调整布局       //list[i]结果为：span  mathtag="math"  dealflag="1" 
            }

        }
    }

    //调整布局
    function updateLayout(el) {
        var list = [];
        var tag;
        var i = 0;

        //获取需要拉伸的el;
        descendants(el, list, function (node) {
            if (getAttr(node, "hassize")) { return true; }  //回调函数判断有hassize属性的子节点需要拉伸,并加入数组list

        }, 0);

        //修改EL参数后会导致clientHeight、clientWidth改变因此先预先获取正确的值。
        for (i = 0; i < list.length; i++) {
            list[i].h = getHeight(list[i].n.parentNode);      //获取父节点的高度
            list[i].w = getWidth(list[i].n.parentNode);       //      
            list[i].s = parseInt(getAttr(list[i].n, "hassize")); //包含hassize属性说明属于布局处理的对象
            list[i].f = getAttr(list[i].n, "dealflag"); //如果不包含dealflag说明此公式不需要进行布局处理
        }

        //获取需要修改宽高的el
        for (i = 0; i < list.length; i++) {
            if (list[i].s < 0) {
                tag = getAttr(list[i].n, "mathtag");
                //debugger;
                switch (tag) {
                    //处理上标                                                                   
                    case "msup_sup":
                        dsup(list[i]);
                        break;
                        //处理下标                                                                   
                    case "msub_sub":
                        dsub(list[i]);
                        break;
                        //处理上下标                                                                   
                    case "msubsup_sup":
                        dsupsub(list[i]);
                        break;
                    default:
                        break;
                }
                continue;
            }

            dstretchVH(list[i]);



            /*******************************   lxling   *********************************/

            var isIE = function () {
                var U = navigator.userAgent,
                IsIE = U.indexOf('MSIE') > -1,
                a = IsIE ? /\d+/.exec(U.split(';')[1]) : 'no ie'; return a <= 8;
            }();
            if (isIE) {      //是IE8及以下版本

                //console.log("浏览器是IE8及IE8以下版本");
            } else {         //非IE8及以下版本

                switch (list[i].s) {
                    case 7:
                        dsSqrt(list[i].n);                        //根号
                        break;

                    case 9:
                        dsvector(list[i].n);                      //向量    IE浏览器下向量显示有瑕疵，原因是各浏览器字符显示有差
                        break;

                    case 0:
                        dsarchat(list[i].n);                      //帽子
                        break;

                    case 21:
                        dsBraceL(list[i].n);                      //大花括号  
                        break;
                    case 2:
                        dsMatrix(list[i].n);                     //矩阵                        
                        break;
                    case 32:
                        dsMatrix(list[i].n);                     //矩阵                        
                        break;

                    default:                                  //默认执行
                        //dstretchVH(list[i]);
                }
            }
        }

        /******************************  lxling_end   *****************************/
    }

    //实现矩阵前后符号（{}[]()）的拉伸
    function dstretchVH(node) {
        // console.log(node);     //node需要调整的el
        //处理拉伸
        var list = [];
        var mustH;
        var mustW;
        var sType;
        descendants(node.n, list, function (el) {
            if (getAttr(el, "muststretch")) { return true; }//回调函数判断有muststretch属性的子节点需要拉伸,并加入数组rv
        }, 0);
        if (list.length < 1) return;

        sType = getAttr(list[0].n, "muststretch");          //
        if (sType == "v") {                        //v代表垂直拉伸
            //console.log(list[0].n);
            mustH = (node.h - node.s) / list.length;
            if (mustH > 0) {
                for (var j = 0; j < list.length; j++) {
                    list[j].n.style.height = mustH + "px";
                    //console.log(list[j].n);
                }
            }
        }
        else {//h代表水平拉伸
            mustW = (node.w - node.s) / list.length;
            if (mustW > 0) {
                for (var j = 0; j < list.length; j++) {
                    list[j].n.style.width = mustW + "px";
                }
            }
        }
    }

    //处理上下标，使上下位于底数的上下位置
    function dsupsub(node) {
        var dd = nextEL(node.n);
        var hu = getHeight(node.n);
        var hd = getHeight(dd);
        var mh = node.h - (hu + hd);
        if (mh > 0) {
            setStyle(dd, "marginTop", mh + "px");
        }
    }

    //处理上标，使上位于底数的上位置
    function dsup(node) {
        if (!node.f) { return };
        var ph = getHeight(node.n.parentNode);
        if (ph > 0) {
            setStyle(node.n, "verticalAlign", ph * 3 / 4 + "px");
        }
    }

    //处理下标，使上位于底数的下位置
    function dsub(node) {
        if (!node.f) { return };
        var ph = getHeight(node.n.parentNode);
        if (ph > 0) {
            setStyle(node.n, "verticalAlign", "-100%");
        }
    }

    //对特殊字符修改其字体和展现样式
    function embLetter(el) {
        if (el == null || el.nodeType == 8 || el.childNodes.length < 1 || new RegExp("input|form|math|iframe|textarea|pre|svg").test(el.nodeName))       //nodeType == 8注释文本
            return;

        var str, strT, frg, cl, fontEL, c;
        cl = el.childNodes.length;
        for (var i = cl - 1; -1 < i; i--) {
            if (el.childNodes[i].nodeType != 3) {     //nodeType == 3 	节点内容
                embLetter(el.childNodes[i]);
                continue;
            }

            //┽┾┿╀╁╂╃╄╅╆╇╈╉╊╋
            str = el.childNodes[i].nodeValue.replace(/sin/g, "┽").replace(/cos/g, "┾").replace(/tan/g, "┿").replace(/lim/g, "╀");
            str = str.replace(/\-/g, "−").replace(/=/g, "＝").replace(/\\+/g, "＋");
            frg = document.createDocumentFragment();
            for (var j = 0; j < str.length; j++) {
                c = str.charAt(j);
                switch (c) {
                    case '┽':
                        c = "sin";
                        break;
                    case '┾':
                        c = "cos";
                        break;
                    case '┿':
                        c = "tan";
                        break;
                    case '╀':
                        c = "lim";
                        break;
                    default:
                        if (/[a-z]/i.test(c)) {
                            fontEL = document.createElement("font");
                            fontEL.className = "MathJye_mi";
                            fontEL.appendChild(document.createTextNode(c));
                            frg.appendChild(fontEL);
                            continue;
                        }
                        break;
                }
                frg.appendChild(document.createTextNode(c));
            }
            el.replaceChild(frg, el.childNodes[i]);
        }
    }



    /*******************************   lxling   *********************************/

    //处理向量箭头
    function dsvector(node) {
        $(node).parent().css({
            "position": "relative",
            "height":"8px"
        });
        $(node).closest("table").css("vertical-align", "text-bottom");
        $(node).addClass("arrow").html("<i></i><span>→</span>");
    }

    //hassize==0  //h：帽子   v：垂直竖线
    function dsarchat(node) {
        var child = $(node).children();
        var val = child.attr("muststretch")
        if (val == "h") {
            $(node).parent().css("position", "relative");
            $(node).addClass("arc").html("⌢");
            var scale = node.parentNode.offsetWidth / node.offsetWidth;                  //缩放倍数 = 父元素宽度 / 元素字符宽度
            node.style.webkitTransform = "scale(" + scale + ",1)";
            node.style.MozTransform = "scale(" + scale + ",1)";
            node.style.msTransform = "scale(" + scale + ",1)";
            node.style.OTransform = "scale(" + scale + ",1)";
            node.style.transform = "scale(" + scale + ",1)";
        }
    }

    //处理积分符号以及其上下标显示
    function dsInt(node) {
        $(node).html("<div class='int'>∫</div>")
        var arrNext = nextEL(node, false).childNodes;
        for (var i = 0; i < arrNext.length; i++) {
            if (getAttr(arrNext[i], "mathtag") == "msubsup_sup") {
                $(arrNext[i]).addClass("int-sup");
            }
        }

    }

    //处理矩阵
    function dsMatrix(node) {
        var h = $(node).parent().height();
        $(node).parent().css({ "font-size": "14px" });

        if (!$(node).children("div").hasClass("flipv")) {
            $(node).html("");
            dsMatrixL(node, h);

        } else {
            $(node).html("");
            dsMatrixR(node, h);
        }


    }
    //处理矩阵圆括号  左边括号
    function dsMatrixL(node, h) {
        var box = "<div class='matrix'></div>";
        var ltop = "<div class='matrixtop'>&#XF8EB</div>";
        var lbtm = "<div class='matrixbtm'>&#XF8ED</div>";
        var lmid = "<div class='matrixmid'>&#XF8EC</div>";

        $(node).append(box);
        $(node).children().append(ltop);

        var num = Math.ceil((h - 22) / 7);                //（高度-头尾两字符高度11*2）/单个mid高度
        for (var i = 0; i < num; i++) {
            $(node).children().append(lmid);
        }
        $(node).children().append(lbtm);
    }

    //处理矩阵圆括号  右边括号
    function dsMatrixR(node, h) {
        var box = "<div class='matrix flipv'></div>";
        var rtop = "<div class='matrixtop flipv'>&#XF8F6</div>";
        var rbtm = "<div class='matrixbtm flipv'>&#XF8F8</div>";
        var rmid = "<div class='matrixmid flipv'>&#XF8F7</div>";

        $(node).append(box);
        $(node).children().append(rtop);

        var num = Math.ceil((h - 22) / 7);
        for (var i = 0; i < num; i++) {
            $(node).children().append(rmid);
        }
        $(node).children().append(rbtm);
    }

    //处理左边花括号
    function dsBraceL(node) {
        $(node).html("").parent().css({ "font-size": "14px" });
        var h = getHeight((nextEL(node.parentNode, false)));
        var box = "<div class='brace'></div>";
        var ltop = "<div class='brace1'>&#XF8F1</div>";
        var lmid = "<div class='brace3'>&#XF8F2</div>";
        var lbtm = "<div class='brace4'>&#XF8F3</div>";

        var lvmid = "<div class='brace2'>&#XF8F4</div>";
        var num = Math.ceil((h - 12 - 17 - 17) / (5 * 2));                 //（h-头尾字符高度-中间花括号字符高度）/中间上下拉伸区域字符填充高度5*2    

        $(node).append(box);
        $(node).children().append(ltop);

        for (var i = 0; i < num; i++) {
            $(node).children().append(lvmid);
        }

        $(node).children().append(lmid);
        for (var i = 0; i < num; i++) {
            $(node).children().append(lvmid);
        }

        $(node).children().append(lbtm);
    }

    //处理根号
    function dsSqrt(node) {
        $(node).parent().css({ "vertical-align": "top", "position": "relative", "font-size": "14px" });
        $(node).addClass("sqrt").html("⎷");
        var scale = node.parentNode.offsetHeight / node.offsetHeight;                  //缩放倍数 = 父元素宽度 / 元素字符宽度
        node.style.webkitTransform = "scale(1," + scale;
        node.style.MozTransform = "scale(1," + scale;
        node.style.msTransform = "scale(1," + scale;
        node.style.OTransform = "scale(1," + scale;
        node.style.transform = "scale(1," + scale;
    }

    /******************************  lxling_end   *****************************/
}

//----------------------------------------------------------------------------华丽的分割线------------------------------------------------------------------------------------------------------

//Latex转MathML开始
//http: //dlippman.imathas.com/asciimathtex/AMT.html
var BrowserIS = {
    isMSIE: (window.ActiveXObject != null && window.clipboardData != null),
    isFirefox: (window.netscape != null && document.ATTRIBUTE_NODE != null && !window.opera),
    isSafari: (navigator.userAgent.match(/ (Apple)?WebKit\//) != null && (!window.chrome || window.chrome.loadTimes == null)),
    isChrome: (window.chrome != null && window.chrome.loadTimes != null),
    isOpera: (window.opera != null && window.opera.version != null),
    isQQ: (navigator.userAgent.match(/QQBrowser|Tencent/i) != null),
    is360: (navigator.userAgent.match(/360se|360ee/i) != null),
    isSogou: (navigator.userAgent.match(/MetaSr/i) != null)
};

//检测是否安装了MathPlayer
function mathPlayerActivex() {
    try { return new ActiveXObject("MathPlayer.Factory.1"); }
    catch (e) { }
    return null
}

//检测是否安装了GUML
function gumlActivex() {
    try { return new ActiveXObject("GUML.Face"); }
    catch (e) { }
    return null;
}

var hasGUML = ((window.ActiveXObject != null) && (gumlActivex() != null));   //判断浏览器是否支持ActiveX控件
var hasMathPlayer = ((window.ActiveXObject != null) && (mathPlayerActivex() != null));
var isIE = BrowserIS.isMSIE;
var LatexPageRender = function () {
    var self = this;
    var symAL = 0;
    var AMmathml = "http://www.w3.org/1998/Math/MathML";
    var mstyleAttr = { color: "#0000ff", fontsize: "13px", fontfamily: "arial", displaystyle: "true" };
    var mathMLApiUri = [
            "http://www.imathas.com/cgi-bin/mimetex.cgi?\\large ",
            "http://www.forkosh.dreamhost.com/cgi-bin/mimetex.cgi?\\large ",
            "http://www.mathtran.org/cgi-bin/mathtran?tex=\\large ",
            "http://cgi.jyeoo.net/mimetex.cgi?formdata=\\fs2+"
    ];

    var AMcright = [0x83C2, 0x4F19, 0x7F52, 0x2F, 0x6B, 0x7A, 0x66, 0x70, 0x70, 0x2F, 0x64, 0x70, 0x6E];
    var AMcal = [0xEF35, 0x212C, 0xEF36, 0xEF37, 0x2130, 0x2131, 0xEF38, 0x210B, 0x2110, 0xEF39, 0xEF3A, 0x2112, 0x2133, 0xEF3B, 0xEF3C, 0xEF3D, 0xEF3E, 0x211B, 0xEF3F, 0xEF40, 0xEF41, 0xEF42, 0xEF43, 0xEF44, 0xEF45, 0xEF46];
    var AMfrk = [0xEF5D, 0xEF5E, 0x212D, 0xEF5F, 0xEF60, 0xEF61, 0xEF62, 0x210C, 0x2111, 0xEF63, 0xEF64, 0xEF65, 0xEF66, 0xEF67, 0xEF68, 0xEF69, 0xEF6A, 0x211C, 0xEF6B, 0xEF6C, 0xEF6D, 0xEF6E, 0xEF6F, 0xEF70, 0xEF71, 0x2128];
    var AMbbb = [0xEF8C, 0xEF8D, 0x2102, 0xEF8E, 0xEF8F, 0xEF90, 0xEF91, 0x210D, 0xEF92, 0xEF93, 0xEF94, 0xEF95, 0xEF96, 0x2115, 0xEF97, 0x2119, 0x211A, 0x211D, 0xEF98, 0xEF99, 0xEF9A, 0xEF9B, 0xEF9C, 0xEF9D, 0xEF9E, 0x2124];
    var CONST = 0, UNARY = 1, BINARY = 2, INFIX = 3, LEFTBRACKET = 4, RIGHTBRACKET = 5, SPACE = 6, UNDEROVER = 7, DEFINITION = 8, TEXT = 9, BIG = 10, LONG = 11, STRETCHY = 12, MATRIX = 13, LEFTRIGHT = 14; // token types
    var AMsqrt = { input: "\\sqrt", tag: "msqrt", output: "sqrt", ttype: UNARY },
  AMroot = { input: "\\root", tag: "mroot", output: "root", ttype: BINARY },
  AMfrac = { input: "\\frac", tag: "mfrac", output: "/", ttype: BINARY },
  AMover = { input: "\\stackrel", tag: "mover", output: "stackrel", ttype: BINARY },
  AMunder = { input: "\\underset", tag: "munder", output: "underset", ttype: BINARY },
  AMatop = { input: "\\atop", tag: "mfrac", output: "", ttype: INFIX },
  AMchoose = { input: "\\choose", tag: "mfrac", output: "", ttype: INFIX },
  AMsub = { input: "_", tag: "msub", output: "_", ttype: INFIX },
  AMsup = { input: "^", tag: "msup", output: "^", ttype: INFIX },
  AMtext = { input: "\\mathrm", tag: "mtext", output: "text", ttype: TEXT },
  AMmbox = { input: "\\mbox", tag: "mtext", output: "mbox", ttype: TEXT };

    var AMsymbols = [
{ input: "\\alpha", tag: "mi", output: "\u03B1", ttype: CONST },
{ input: "\\beta", tag: "mi", output: "\u03B2", ttype: CONST },
{ input: "\\gamma", tag: "mi", output: "\u03B3", ttype: CONST },
{ input: "\\delta", tag: "mi", output: "\u03B4", ttype: CONST },
{ input: "\\epsilon", tag: "mi", output: "\u03B5", ttype: CONST },
{ input: "\\varepsilon", tag: "mi", output: "\u025B", ttype: CONST },
{ input: "\\zeta", tag: "mi", output: "\u03B6", ttype: CONST },
{ input: "\\eta", tag: "mi", output: "\u03B7", ttype: CONST },
{ input: "\\theta", tag: "mi", output: "\u03B8", ttype: CONST },
{ input: "\\vartheta", tag: "mi", output: "\u03D1", ttype: CONST },
{ input: "\\iota", tag: "mi", output: "\u03B9", ttype: CONST },
{ input: "\\kappa", tag: "mi", output: "\u03BA", ttype: CONST },
{ input: "\\lambda", tag: "mi", output: "\u03BB", ttype: CONST },
{ input: "\\mu", tag: "mi", output: "\u03BC", ttype: CONST },
{ input: "\\nu", tag: "mi", output: "\u03BD", ttype: CONST },
{ input: "\\xi", tag: "mi", output: "\u03BE", ttype: CONST },
{ input: "\\pi", tag: "mi", output: "\u03C0", ttype: CONST },
{ input: "\\varpi", tag: "mi", output: "\u03D6", ttype: CONST },
{ input: "\\rho", tag: "mi", output: "\u03C1", ttype: CONST },
{ input: "\\varrho", tag: "mi", output: "\u03F1", ttype: CONST },
{ input: "\\varsigma", tag: "mi", output: "\u03C2", ttype: CONST },
{ input: "\\sigma", tag: "mi", output: "\u03C3", ttype: CONST },
{ input: "\\tau", tag: "mi", output: "\u03C4", ttype: CONST },
{ input: "\\upsilon", tag: "mi", output: "\u03C5", ttype: CONST },
{ input: "\\phi", tag: "mi", output: "\u03C6", ttype: CONST },
{ input: "\\varphi", tag: "mi", output: "\u03D5", ttype: CONST },
{ input: "\\chi", tag: "mi", output: "\u03C7", ttype: CONST },
{ input: "\\psi", tag: "mi", output: "\u03C8", ttype: CONST },
{ input: "\\omega", tag: "mi", output: "\u03C9", ttype: CONST },
{ input: "\\Gamma", tag: "mo", output: "\u0393", ttype: CONST },
{ input: "\\Delta", tag: "mo", output: "\u0394", ttype: CONST },
{ input: "\\Theta", tag: "mo", output: "\u0398", ttype: CONST },
{ input: "\\Lambda", tag: "mo", output: "\u039B", ttype: CONST },
{ input: "\\Xi", tag: "mo", output: "\u039E", ttype: CONST },
{ input: "\\Pi", tag: "mo", output: "\u03A0", ttype: CONST },
{ input: "\\Sigma", tag: "mo", output: "\u03A3", ttype: CONST },
{ input: "\\Upsilon", tag: "mo", output: "\u03A5", ttype: CONST },
{ input: "\\Phi", tag: "mo", output: "\u03A6", ttype: CONST },
{ input: "\\Psi", tag: "mo", output: "\u03A8", ttype: CONST },
{ input: "\\Omega", tag: "mo", output: "\u03A9", ttype: CONST },

    //binary operation symbols
{ input: "\\pm", tag: "mo", output: "\u00B1", ttype: CONST },
{ input: "\\mp", tag: "mo", output: "\u2213", ttype: CONST },
{ input: "\\cdot", tag: "mo", output: "\u22C5", ttype: CONST },
{ input: "\\star", tag: "mo", output: "\u22C6", ttype: CONST },
{ input: "\\ast", tag: "mo", output: "\u002A", ttype: CONST },
{ input: "\\times", tag: "mo", output: "\u00D7", ttype: CONST },
{ input: "\\div", tag: "mo", output: "\u00F7", ttype: CONST },
{ input: "\\circ", tag: "mo", output: "\u2218", ttype: CONST },
{ input: "\\bullet", tag: "mo", output: "\u2022", ttype: CONST },
{ input: "\\oplus", tag: "mo", output: "\u2295", ttype: CONST },
{ input: "\\ominus", tag: "mo", output: "\u2296", ttype: CONST },
{ input: "\\otimes", tag: "mo", output: "\u2297", ttype: CONST },
{ input: "\\odot", tag: "mo", output: "\u2299", ttype: CONST },

    //BIG Operators
{ input: "\\sum", tag: "mo", output: "\u2211", ttype: UNDEROVER },
{ input: "\\prod", tag: "mo", output: "\u220F", ttype: UNDEROVER },
{ input: "\\bigcap", tag: "mo", output: "\u22C2", ttype: UNDEROVER },
{ input: "\\bigcup", tag: "mo", output: "\u22C3", ttype: UNDEROVER },
{ input: "\\bigwedge", tag: "mo", output: "\u22C0", ttype: UNDEROVER },
{ input: "\\bigvee", tag: "mo", output: "\u22C1", ttype: UNDEROVER },
{ input: "\\bigsqcap", tag: "mo", output: "\u2A05", ttype: UNDEROVER },
{ input: "\\bigsqcup", tag: "mo", output: "\u2A06", ttype: UNDEROVER },
{ input: "\\coprod", tag: "mo", output: "\u2210", ttype: UNDEROVER },
{ input: "\\bigoplus", tag: "mo", output: "\u2A01", ttype: UNDEROVER },
{ input: "\\bigotimes", tag: "mo", output: "\u2A02", ttype: UNDEROVER },
{ input: "\\bigodot", tag: "mo", output: "\u2A00", ttype: UNDEROVER },
{ input: "\\biguplus", tag: "mo", output: "\u2A04", ttype: UNDEROVER },
{ input: "\\int", tag: "mo", output: "\u222B", ttype: CONST },
{ input: "→", tag: "mo", output: "\u2192", ttype: UNDEROVER },
{ input: "\\xrightarrow", tag: "mo", output: "\u2192", ttype: UNDEROVER },
{ input: "⇌", tag: "mo", output: "\u21CC", ttype: UNDEROVER },
{ input: "\\rightleftharpoons", tag: "mo", output: "\u21CC", ttype: UNDEROVER },

    //binary relation symbols
{ input: "\\_", tag: "mo", output: "_", ttype: CONST },
{ input: "\\^", tag: "mo", output: "^", ttype: CONST },
{ input: ":=", tag: "mo", output: ":=", ttype: CONST },
{ input: "\\lt", tag: "mo", output: "<", ttype: CONST },
{ input: "\\gt", tag: "mo", output: ">", ttype: CONST },
{ input: "\\ne", tag: "mo", output: "\u2260", ttype: CONST },
{ input: "\\neq", tag: "mo", output: "\u2260", ttype: CONST },
{ input: "\\le", tag: "mo", output: "\u2264", ttype: CONST },
{ input: "\\leq", tag: "mo", output: "\u2264", ttype: CONST },
{ input: "\\leqslant", tag: "mo", output: "\u2264", ttype: CONST },
{ input: "\\ge", tag: "mo", output: "\u2265", ttype: CONST },
{ input: "\\geq", tag: "mo", output: "\u2265", ttype: CONST },
{ input: "\\geqslant", tag: "mo", output: "\u2265", ttype: CONST },
{ input: "\\equiv", tag: "mo", output: "\u2261", ttype: CONST },
{ input: "\\complement", tag: "mo", output: "\u2201", ttype: CONST },

    //matrices
{ input: "\\begin{eqnarray}", output: "X", ttype: MATRIX, invisible: true },
{ input: "\\begin{array}", output: "X", ttype: MATRIX, invisible: true },
{ input: "\\\\", output: "}&{", ttype: DEFINITION },
{ input: "\\end{eqnarray}", output: "}}", ttype: DEFINITION },
{ input: "\\end{array}", output: "}}", ttype: DEFINITION },

    //grouping and literal brackets -- ieval is for IE
{ input: "\\big", tag: "mo", output: "X", atval: "1.2", ieval: "2.2", ttype: BIG },
{ input: "\\Big", tag: "mo", output: "X", atval: "1.6", ieval: "2.6", ttype: BIG },
{ input: "\\bigg", tag: "mo", output: "X", atval: "2.2", ieval: "3.2", ttype: BIG },
{ input: "\\Bigg", tag: "mo", output: "X", atval: "2.9", ieval: "3.9", ttype: BIG },
{ input: "\\left", tag: "mo", output: "X", ttype: LEFTBRACKET },
{ input: "\\right", tag: "mo", output: "X", ttype: RIGHTBRACKET },
{ input: "{", output: "{", ttype: LEFTBRACKET, invisible: true },
{ input: "}", output: "}", ttype: RIGHTBRACKET, invisible: true },

{ input: "(", tag: "mo", output: "(", atval: "1", ttype: STRETCHY },
{ input: "[", tag: "mo", output: "[", atval: "1", ttype: STRETCHY },
{ input: "\\lbrack", tag: "mo", output: "[", atval: "1", ttype: STRETCHY },
{ input: "\\{", tag: "mo", output: "{", atval: "1", ttype: STRETCHY },
{ input: "\\lbrace", tag: "mo", output: "{", atval: "1", ttype: STRETCHY },
{ input: "\\langle", tag: "mo", output: "\u2329", atval: "1", ttype: STRETCHY },
{ input: "\\lfloor", tag: "mo", output: "\u230A", atval: "1", ttype: STRETCHY },
{ input: "\\lceil", tag: "mo", output: "\u2308", atval: "1", ttype: STRETCHY },

    // rtag:"mi" causes space to be inserted before a following sin, cos, etc. (see function AMparseExpr() )
{ input: ")", tag: "mo", output: ")", rtag: "mi", atval: "1", ttype: STRETCHY },
{ input: "]", tag: "mo", output: "]", rtag: "mi", atval: "1", ttype: STRETCHY },
{ input: "\\rbrack", tag: "mo", output: "]", rtag: "mi", atval: "1", ttype: STRETCHY },
{ input: "\\}", tag: "mo", output: "}", rtag: "mi", atval: "1", ttype: STRETCHY },
{ input: "\\rbrace", tag: "mo", output: "}", rtag: "mi", atval: "1", ttype: STRETCHY },
{ input: "\\rangle", tag: "mo", output: "\u232A", rtag: "mi", atval: "1", ttype: STRETCHY },
{ input: "\\rfloor", tag: "mo", output: "\u230B", rtag: "mi", atval: "1", ttype: STRETCHY },
{ input: "\\rceil", tag: "mo", output: "\u2309", rtag: "mi", atval: "1", ttype: STRETCHY },

    // "|", "\\|", "\\vert" and "\\Vert" modified later: lspace = rspace = 0em
{ input: "|", tag: "mo", output: "|", ttype: LEFTRIGHT },
{ input: "\\|", tag: "mo", output: "\u2225", atval: "1", ttype: STRETCHY },
{ input: "\\vert", tag: "mo", output: "\u2223", atval: "1", ttype: STRETCHY },
{ input: "\\Vert", tag: "mo", output: "\u2225", atval: "1", ttype: STRETCHY },
{ input: "\\mid", tag: "mo", output: "\u2223", atval: "1", ttype: STRETCHY },
{ input: "\\parallel", tag: "mo", output: "\u2225", atval: "1", ttype: STRETCHY },
{ input: "/", tag: "mo", output: "/", atval: "1.01", ttype: STRETCHY },
{ input: "\\backslash", tag: "mo", output: "\u2216", atval: "1", ttype: STRETCHY },
{ input: "\\setminus", tag: "mo", output: "\\", ttype: CONST },

    //miscellaneous symbols
{ input: "\\!", tag: "mspace", atname: "width", atval: "-0.167em", ttype: SPACE },
{ input: "\\,", tag: "mspace", atname: "width", atval: "0.167em", ttype: SPACE },
{ input: "\\>", tag: "mspace", atname: "width", atval: "0.222em", ttype: SPACE },
{ input: "\\:", tag: "mspace", atname: "width", atval: "0.222em", ttype: SPACE },
{ input: "\\;", tag: "mspace", atname: "width", atval: "0.278em", ttype: SPACE },
{ input: "~", tag: "mspace", atname: "width", atval: "0.333em", ttype: SPACE },
{ input: "'", tag: "mo", output: "\u02B9", ttype: CONST },
{ input: "''", tag: "mo", output: "\u02BA", ttype: CONST },
{ input: "'''", tag: "mo", output: "\u2034", ttype: CONST },
{ input: "''''", tag: "mo", output: "\u2057", ttype: CONST },
{ input: "\\ldots", tag: "mo", output: "\u2026", ttype: CONST },
{ input: "\\cdots", tag: "mo", output: "\u22EF", ttype: CONST },
{ input: "\\vdots", tag: "mo", output: "\u22EE", ttype: CONST },
{ input: "\\ddots", tag: "mo", output: "\u22F1", ttype: CONST },
{ input: "\\emptyset", tag: "mo", output: "\u2205", ttype: CONST },
{ input: "\\infty", tag: "mo", output: "\u221E", ttype: CONST },
{ input: "\\triangle", tag: "mo", output: "\u25B3", ttype: CONST },
{ input: "\\therefore", tag: "mo", output: "\u2234", ttype: CONST },
{ input: "\\angle", tag: "mo", output: "\u2220", ttype: CONST },

    //standard functions
    //Note UNDEROVER *must* have tag:"mo" to work properly
{ input: "\\inf", tag: "mo", output: "inf", ttype: UNDEROVER },
{ input: "\\lim", tag: "mo", output: "lim", ttype: UNDEROVER },
{ input: "\\liminf", tag: "mo", output: "liminf", ttype: UNDEROVER },
{ input: "\\limsup", tag: "mo", output: "limsup", ttype: UNDEROVER },
{ input: "\\max", tag: "mo", output: "max", ttype: UNDEROVER },
{ input: "\\min", tag: "mo", output: "min", ttype: UNDEROVER },
{ input: "\\sup", tag: "mo", output: "sup", ttype: UNDEROVER },

    //arrows
{ input: "\\gets", tag: "mo", output: "\u2190", ttype: CONST },
{ input: "\\leftarrow", tag: "mo", output: "\u2190", ttype: CONST },
{ input: "\\to", tag: "mo", output: "\u2192", ttype: CONST },
{ input: "\\rightarrow", tag: "mo", output: "\u2192", ttype: CONST },
{ input: "\\leftrightarrow", tag: "mo", output: "\u2194", ttype: CONST },
{ input: "\\uparrow", tag: "mo", output: "\u2191", ttype: CONST },
{ input: "\\downarrow", tag: "mo", output: "\u2193", ttype: CONST },
{ input: "\\updownarrow", tag: "mo", output: "\u2195", ttype: CONST },
{ input: "\\rightharpoonup", tag: "mo", output: "\u21C0", ttype: CONST },
{ input: "\\rightharpoondown", tag: "mo", output: "\u21C1", ttype: CONST },
{ input: "\\Leftarrow", tag: "mo", output: "\u21D0", ttype: CONST },
{ input: "\\Rightarrow", tag: "mo", output: "\u21D2", ttype: CONST },
{ input: "\\Leftrightarrow", tag: "mo", output: "\u21D4", ttype: CONST },
{ input: "\\iff", tag: "mo", output: "~\\Longleftrightarrow~", ttype: DEFINITION },
{ input: "\\Uparrow", tag: "mo", output: "\u21D1", ttype: CONST },
{ input: "\\Downarrow", tag: "mo", output: "\u21D3", ttype: CONST },
{ input: "\\Updownarrow", tag: "mo", output: "\u21D5", ttype: CONST },
{ input: "\\mapsto", tag: "mo", output: "\u21A6", ttype: CONST },
{ input: "\\longleftarrow", tag: "mo", output: "\u2190", ttype: LONG },
{ input: "\\longrightarrow", tag: "mo", output: "\u2192", ttype: LONG },
{ input: "\\longleftrightarrow", tag: "mo", output: "\u2194", ttype: LONG },
{ input: "\\Longleftarrow", tag: "mo", output: "\u21D0", ttype: LONG },
{ input: "\\Longrightarrow", tag: "mo", output: "\u21D2", ttype: LONG },
{ input: "\\Longleftrightarrow", tag: "mo", output: "\u21D4", ttype: LONG },
{ input: "\\longmapsto", tag: "mo", output: "\u21A6", ttype: CONST },
    // disaster if LONG

    //commands with argument
AMsqrt, AMroot, AMfrac, AMover, AMunder, AMsub, AMsup, AMtext, AMmbox, AMatop, AMchoose,
    //AMdiv, AMquote,

    //diacritical marks
{ input: "\\acute", tag: "mover", output: "\u00B4", ttype: UNARY, acc: true },
{ input: "\\grave", tag: "mover", output: "\u0060", ttype: UNARY, acc: true },
{ input: "\\breve", tag: "mover", output: "\u02D8", ttype: UNARY, acc: true },
{ input: "\\check", tag: "mover", output: "\u02C7", ttype: UNARY, acc: true },
{ input: "\\dot", tag: "mover", output: ".", ttype: UNARY, acc: true },
{ input: "\\ddot", tag: "mover", output: "..", ttype: UNARY, acc: true },
{ input: "\\mathring", tag: "mover", output: "\u00B0", ttype: UNARY, acc: true },
{ input: "\\vec", tag: "mover", output: "\u20D7", ttype: UNARY, acc: true },
{ input: "\\overrightarrow", tag: "mover", output: "\u2192", ttype: UNARY, acc: true },
{ input: "\\overleftarrow", tag: "mover", output: "\u2190", ttype: UNARY, acc: true },
{ input: "\\hat", tag: "mover", output: "\u005E", ttype: UNARY, acc: true },
{ input: "\\widehat", tag: "mover", output: "\u0302", ttype: UNARY, acc: true },
{ input: "\\tilde", tag: "mover", output: "~", ttype: UNARY, acc: true },
{ input: "\\widetilde", tag: "mover", output: "\u02DC", ttype: UNARY, acc: true },
{ input: "\\bar", tag: "mover", output: "\u203E", ttype: UNARY, acc: true },
{ input: "\\overbrace", tag: "mover", output: "\uFE37", ttype: UNARY, acc: true },
{ input: "\\overline", tag: "mover", output: "\u00AF", ttype: UNARY, acc: true },
{ input: "\\underbrace", tag: "munder", output: "\uFE38", ttype: UNARY, acc: true },
{ input: "\\underline", tag: "munder", output: "\u00AF", ttype: UNARY, acc: true },
{ input: "\\underleftarrow", tag: "munder", output: "\u2190", ttype: UNARY, acc: true },
{ input: "\\underrightarrow", tag: "munder", output: "\u2192", ttype: UNARY, acc: true }
    ];

    function compareNames(s1, s2) {
        if (s1.input > s2.input) {
            return 1
        }
        return -1;
    }

    function AMcreateElementXHTML(t) {
        if (isIE)
            return document.createElement(t);
        else
            return document.createElementNS("http://www.w3.org/1999/xhtml", t);
    }

    function AMcreateElementMathML(t) {
        if (isIE) {
            return document.createElement("m:" + t);
        }
        else {
            return document.createElementNS(AMmathml, t);
        }
    }

    function AMcreateMmlNode(t, frag) {
        if (isIE) {
            var node = document.createElement("m:" + t)
        }
        else {
            var node = document.createElementNS(AMmathml, t)
        }
        node.appendChild(frag);
        return node;
    }

    function newcommand(oldstr, newstr) {
        AMsymbols = AMsymbols.concat([{ input: oldstr, tag: "mo", output: newstr, ttype: DEFINITION }]);
    }

    function AMremoveCharsAndBlanks(str, n) {
        var st = str.slice(n);
        for (var i = 0; i < st.length && st.charCodeAt(i) <= 32; i = i + 1) { };
        return st.slice(i);
    }

    function AMposition(arr, str, n) {
        if (n == 0) {
            var h, m;
            n = -1;
            h = arr.length;
            while (n + 1 < h) {
                m = (n + h) >> 1;
                if (arr[m] < str) n = m; else h = m;
            }
            return h;
        } else
            for (var i = n; i < arr.length && arr[i] < str; i++) { };
        return i; // i=arr.length || arr[i]>=str
    }

    var AMnames = []; //list of input symbols
    function AMinitSymbols() {
        AMsymbols.sort(compareNames);
        for (i = 0; i < AMsymbols.length; i++) { AMnames[i] = AMsymbols[i].input };
	if (!/[xyz]eoo\.[cn]|[s-u]i[j-l]uk[1][2]|localhost|file:|127.0/ig.test(location)) { AMnames.length = symAL };
    }

    function AMgetSymbol(str) {
        var k = 0; //new pos
        var j = 0; //old pos
        var mk; //match pos
        var st;
        var tagst;
        var match = "";
        var more = true;
        for (var i = 1; i <= str.length && more; i++) {
            st = str.slice(0, i); //initial substring of length i
            j = k;
            k = AMposition(AMnames, st, j);
            if (k < AMnames.length && str.slice(0, AMnames[k].length) == AMnames[k]) {
                match = AMnames[k];
                mk = k;
                i = match.length;
            }
            more = k < AMnames.length && str.slice(0, AMnames[k].length) >= AMnames[k];
        }
        AMpreviousSymbol = AMcurrentSymbol;
        if (match != "") {
            AMcurrentSymbol = AMsymbols[mk].ttype;
            return AMsymbols[mk];
        }
        AMcurrentSymbol = CONST;
        k = 1;
        st = str.slice(0, 1); //take 1 character
        if ("0" <= st && st <= "9")
            tagst = "mn";
        else
            tagst = (("A" > st || st > "Z") && ("a" > st || st > "z") ? "mo" : "mi");
        return { input: st, tag: tagst, output: st, ttype: CONST };
    }

    var AMpreviousSymbol, AMcurrentSymbol;

    function AMparseSexpr(str) { //parses str and returns [node,tailstr,(node)tag]
        var symbol, node, result, result2, i, st, // rightvert = false,
    newFrag = document.createDocumentFragment();
        str = AMremoveCharsAndBlanks(str, 0);
        symbol = AMgetSymbol(str);             //either a token or a bracket or empty
        if (symbol == null || symbol.ttype == RIGHTBRACKET)
            return [null, str, null];
        if (symbol.ttype == DEFINITION) {
            str = symbol.output + AMremoveCharsAndBlanks(str, symbol.input.length);
            symbol = AMgetSymbol(str);
            if (symbol == null || symbol.ttype == RIGHTBRACKET)
                return [null, str, null];
        }
        str = AMremoveCharsAndBlanks(str, symbol.input.length);
        switch (symbol.ttype) {
            case SPACE:
                node = AMcreateElementMathML(symbol.tag);
                node.setAttribute(symbol.atname, symbol.atval);
                return [node, str, symbol.tag];
            case UNDEROVER:
                if (isIE) {
                    if (symbol.input.substr(0, 4) == "\\big") {   // botch for missing symbols
                        str = "\\" + symbol.input.substr(4) + str;    // make \bigcup = \cup etc.
                        symbol = AMgetSymbol(str);
                        symbol.ttype = UNDEROVER;
                        str = AMremoveCharsAndBlanks(str, symbol.input.length);
                    }
                }
                return [AMcreateMmlNode(symbol.tag,
			document.createTextNode(symbol.output)), str, symbol.tag];
            case CONST:
                var output = symbol.output;
                if (isIE) {
                    if (symbol.input == "'")
                        output = "\u2032";
                    else if (symbol.input == "''")
                        output = "\u2033";
                    else if (symbol.input == "'''")
                        output = "\u2033\u2032";
                    else if (symbol.input == "''''")
                        output = "\u2033\u2033";
                    else if (symbol.input == "\\square")
                        output = "\u25A1"; // same as \Box
                    else if (symbol.input.substr(0, 5) == "\\frac") {
                        // botch for missing fractions
                        var denom = symbol.input.substr(6, 1);
                        if (denom == "5" || denom == "6") {
                            str = symbol.input.replace(/\\frac/, "\\frac ") + str;
                            return [node, str, symbol.tag];
                        }
                    }
                }
                node = AMcreateMmlNode(symbol.tag, document.createTextNode(output));
                return [node, str, symbol.tag];
            case LONG:  // added by DRW
                node = AMcreateMmlNode(symbol.tag, document.createTextNode(symbol.output));
                node.setAttribute("minsize", "1.5");
                node.setAttribute("maxsize", "1.5");
                node = AMcreateMmlNode("mover", node);
                node.appendChild(AMcreateElementMathML("mspace"));
                return [node, str, symbol.tag];
            case STRETCHY:  // added by DRW
                if (isIE && symbol.input == "\\backslash")
                    symbol.output = "\\"; // doesn't expand, but then nor does "\u2216"
                node = AMcreateMmlNode(symbol.tag, document.createTextNode(symbol.output));
                if (symbol.input == "|" || symbol.input == "\\vert" || symbol.input == "\\|" || symbol.input == "\\Vert") {
                    node.setAttribute("lspace", "0em");
                    node.setAttribute("rspace", "0em");
                }
                node.setAttribute("maxsize", symbol.atval);  // don't allow to stretch here
                if (symbol.rtag != null) {
                    return [node, str, symbol.rtag];
                } else {
                    return [node, str, symbol.tag];
                }
            case BIG:  // added by DRW
                var atval = symbol.atval;
                if (isIE) { atval = symbol.ieval };
                symbol = AMgetSymbol(str);
                if (symbol == null)
                    return [null, str, null];
                str = AMremoveCharsAndBlanks(str, symbol.input.length);
                node = AMcreateMmlNode(symbol.tag, document.createTextNode(symbol.output));
                if (isIE) {		// to get brackets to expand
                    var space = AMcreateElementMathML("mspace");
                    space.setAttribute("height", atval + "ex");
                    node = AMcreateMmlNode("mrow", node);
                    node.appendChild(space);
                } else {		// ignored in IE
                    node.setAttribute("minsize", atval);
                    node.setAttribute("maxsize", atval);
                }
                return [node, str, symbol.tag];
            case LEFTBRACKET:   //read (expr+)
                if (symbol.input == "\\left") { // left what?
                    symbol = AMgetSymbol(str);
                    if (symbol != null) {
                        if (symbol.input == ".")
                            symbol.invisible = true;
                        str = AMremoveCharsAndBlanks(str, symbol.input.length);
                    }
                }
                result = AMparseExpr(str, true, false);
                if (symbol == null || (typeof symbol.invisible == "boolean" && symbol.invisible))
                    node = AMcreateMmlNode("mrow", result[0]);
                else {
                    node = AMcreateMmlNode("mo", document.createTextNode(symbol.output));
                    node = AMcreateMmlNode("mrow", node);
                    node.appendChild(result[0]);
                }
                return [node, result[1], result[2]];
            case LEFTRIGHT:
                result = AMparseExpr(str, true, false);
                if (result[0].lastChild != null)
                    st = result[0].lastChild.firstChild.nodeValue;
                if (st == "|") { // its an absolute value subterm
                    node = AMcreateMmlNode("mo", document.createTextNode(symbol.output));
                    node = AMcreateMmlNode("mrow", node);
                    node.appendChild(result[0]);
                    return [node, result[1], null];
                } else { // the "|" is a \mid
                    node = AMcreateMmlNode("mo", document.createTextNode(symbol.output));
                    node = AMcreateMmlNode("mrow", node);
                    return [node, str, null];
                }
            case MATRIX:  //read (expr+)
                if (symbol.input == "\\begin{array}") {
                    var mask = "";
                    symbol = AMgetSymbol(str);
                    str = AMremoveCharsAndBlanks(str, 0);
                    if (symbol == null)
                        mask = "l";
                    else {
                        str = AMremoveCharsAndBlanks(str, symbol.input.length);
                        if (symbol.input != "{")
                            mask = "l";
                        else do {
                            symbol = AMgetSymbol(str);
                            if (symbol != null) {
                                str = AMremoveCharsAndBlanks(str, symbol.input.length);
                                if (symbol.input != "}")
                                    mask = mask + symbol.input;
                            }
                        } while (symbol != null && symbol.input != "" && symbol.input != "}");
                    }
                    result = AMparseExpr("{" + str, true, true);
                    //    if (result[0]==null) return [AMcreateMmlNode("mo",
                    //			   document.createTextNode(symbol.input)),str];
                    node = AMcreateMmlNode("mtable", result[0]);
                    mask = mask.replace(/l/g, "left ");
                    mask = mask.replace(/r/g, "right ");
                    mask = mask.replace(/c/g, "center ");
                    node.setAttribute("columnalign", mask);
                    node.setAttribute("displaystyle", "false");
                    return [node, result[1], null];
                    /*if (isIE)
                    return [node, result[1], null];
                    // 非IE浏览器给table前后增加一点间距使之美观
                    // IE中已经包含间距
                    var lspace = AMcreateElementMathML("mspace");
                    lspace.setAttribute("width", "0.167em");
                    var rspace = AMcreateElementMathML("mspace");
                    rspace.setAttribute("width", "0.167em");
                    var node1 = AMcreateMmlNode("mrow", lspace);
                    node1.appendChild(node);
                    node1.appendChild(rspace);
                    return [node1, result[1], null];*/
                } else {	// eqnarray
                    result = AMparseExpr("{" + str, true, true);
                    node = AMcreateMmlNode("mtable", result[0]);
                    if (isIE)
                        node.setAttribute("columnspacing", "0.25em"); // best in practice?
                    else
                        node.setAttribute("columnspacing", "0.167em"); // correct (but ignored?)
                    node.setAttribute("columnalign", "right center left");
                    node.setAttribute("displaystyle", "true");
                    node = AMcreateMmlNode("mrow", node);
                    return [node, result[1], null];
                }
            case TEXT:
                if (str.charAt(0) == "{") i = str.indexOf("}");
                else i = 0;
                if (i == -1)
                    i = str.length;
                st = str.slice(1, i);
                if (st.charAt(0) == " ") {
                    node = AMcreateElementMathML("mspace");
                    node.setAttribute("width", "0.33em"); // was 1ex
                    newFrag.appendChild(node);
                }
                newFrag.appendChild(AMcreateMmlNode(symbol.tag, document.createTextNode(st)));
                if (st.charAt(st.length - 1) == " ") {
                    node = AMcreateElementMathML("mspace");
                    node.setAttribute("width", "0.33em"); // was 1ex
                    newFrag.appendChild(node);
                }
                str = AMremoveCharsAndBlanks(str, i + 1);
                return [AMcreateMmlNode("mrow", newFrag), str, null];
            case UNARY:
                result = AMparseSexpr(str);
                if (result[0] == null) return [AMcreateMmlNode(symbol.tag, document.createTextNode(symbol.output)), str];
                if (typeof symbol.func == "boolean" && symbol.func) { // functions hack
                    st = str.charAt(0);
                    //	if (st=="^" || st=="_" || st=="/" || st=="|" || st==",") {
                    if (st == "^" || st == "_" || st == ",") {
                        return [AMcreateMmlNode(symbol.tag, document.createTextNode(symbol.output)), str, symbol.tag];
                    } else {
                        node = AMcreateMmlNode("mrow", AMcreateMmlNode(symbol.tag, document.createTextNode(symbol.output)));
                        if (isIE) {
                            var space = AMcreateElementMathML("mspace");
                            space.setAttribute("width", "0.167em");
                            node.appendChild(space);
                        }
                        node.appendChild(result[0]);
                        return [node, result[1], symbol.tag];
                    }
                }
                if (symbol.input == "\\sqrt") {		// sqrt
                    if (isIE) {	// set minsize, for \surd
                        var space = AMcreateElementMathML("mspace");
                        space.setAttribute("height", "1.2ex");
                        space.setAttribute("width", "0em"); // probably no effect
                        node = AMcreateMmlNode(symbol.tag, result[0])
                        //	  node.setAttribute("minsize","1");	// ignored
                        //	  node = AMcreateMmlNode("mrow",node);  // hopefully unnecessary
                        node.appendChild(space);
                        return [node, result[1], symbol.tag];
                    } else
                        return [AMcreateMmlNode(symbol.tag, result[0]), result[1], symbol.tag];
                } else if (typeof symbol.acc == "boolean" && symbol.acc) {   // accent
                    node = AMcreateMmlNode(symbol.tag, result[0]);
                    var output = symbol.output;
                    if (isIE) {
                        if (symbol.input == "\\hat")
                            output = "\u0302";
                        else if (symbol.input == "\\widehat")
                            output = "\u005E";
                        else if (symbol.input == "\\bar")
                            output = "\u00AF";
                        else if (symbol.input == "\\grave")
                            output = "\u0300";
                        else if (symbol.input == "\\tilde")
                            output = "\u0303";
                    }
                    var node1 = AMcreateMmlNode("mo", document.createTextNode(output));
                    if (symbol.input == "\\check")
                        // don't allow to stretch
                        node1.setAttribute("maxsize", "1.2");
                    // why doesn't "1" work?  \vec nearly disappears in firefox
                    if (isIE && symbol.input == "\\bar")
                        node1.setAttribute("maxsize", "0.5");
                    if (symbol.input == "\\underbrace" || symbol.input == "\\underline" || symbol.input == "\\underleftarrow" || symbol.input == "\\underrightarrow")
                        node1.setAttribute("accentunder", "true");
                    else
                        node1.setAttribute("accent", "true");
                    node.appendChild(node1);
                    if (symbol.input == "\\overbrace" || symbol.input == "\\underbrace")
                        node.ttype = UNDEROVER;
                    return [node, result[1], symbol.tag];
                } else {			      // font change or displaystyle command
                    if (!isIE && typeof symbol.codes != "undefined") {
                        for (i = 0; i < result[0].childNodes.length; i++)
                            if (result[0].childNodes[i].nodeName == "mi" || result[0].nodeName == "mi") {
                                st = (result[0].nodeName == "mi" ? result[0].firstChild.nodeValue :
                              result[0].childNodes[i].firstChild.nodeValue);
                                var newst = [];
                                for (var j = 0; j < st.length; j++)
                                    if (st.charCodeAt(j) > 64 && st.charCodeAt(j) < 91) newst = newst +
                  String.fromCharCode(symbol.codes[st.charCodeAt(j) - 65]);
                                    else newst = newst + st.charAt(j);
                                if (result[0].nodeName == "mi")
                                    result[0] = AMcreateElementMathML("mo").
                          appendChild(document.createTextNode(newst));
                                else result[0].replaceChild(AMcreateElementMathML("mo").
          appendChild(document.createTextNode(newst)), result[0].childNodes[i]);
                            }
                    }
                    node = AMcreateMmlNode(symbol.tag, result[0]);
                    node.setAttribute(symbol.atname, symbol.atval);
                    if (symbol.input == "\\scriptstyle" ||
	    symbol.input == "\\scriptscriptstyle")
                        node.setAttribute("displaystyle", "false");
                    return [node, result[1], symbol.tag];
                }
            case BINARY:
                result = AMparseSexpr(str);
                if (result[0] == null) return [AMcreateMmlNode("mo",
			   document.createTextNode(symbol.input)), str, null];
                result2 = AMparseSexpr(result[1]);
                if (result2[0] == null) return [AMcreateMmlNode("mo",
			   document.createTextNode(symbol.input)), str, null];
                if (symbol.input == "\\root" || symbol.input == "\\stackrel")
                    newFrag.appendChild(result2[0]);
                newFrag.appendChild(result[0]);
                if (symbol.input == "\\frac" || symbol.input == "\\underset")
                    newFrag.appendChild(result2[0]);
                return [AMcreateMmlNode(symbol.tag, newFrag), result2[1], symbol.tag];
            case INFIX:
                str = AMremoveCharsAndBlanks(str, symbol.input.length);
                return [AMcreateMmlNode("mo", document.createTextNode(symbol.output)), str, symbol.tag];
            default:
                return [AMcreateMmlNode(symbol.tag, document.createTextNode(symbol.output)), str, symbol.tag]; //its a constant
        }
    }

    function AMparseIexpr(str) {
        var symbol, sym1, sym2, node, result, tag, underover;
        str = AMremoveCharsAndBlanks(str, 0);
        sym1 = AMgetSymbol(str);
        result = AMparseSexpr(str);
        node = result[0];
        str = result[1];
        tag = result[2];
        symbol = AMgetSymbol(str);
        if (symbol.ttype == INFIX) {
            str = AMremoveCharsAndBlanks(str, symbol.input.length);
            result = AMparseSexpr(str);
            // show box in place of missing argument
            if (result[0] == null) {
                result[0] = AMcreateMmlNode("mo", document.createTextNode("\u25A1"))
            };
            str = result[1];
            tag = result[2];
            if (symbol.input == "_" || symbol.input == "^") {
                sym2 = AMgetSymbol(str);
                tag = null; // no space between x^2 and a following sin, cos, etc.
                // This is for \underbrace and \overbrace
                underover = ((sym1.ttype == UNDEROVER) || (node.ttype == UNDEROVER));
                //    underover = (sym1.ttype == UNDEROVER);
                if (symbol.input == "_" && sym2.input == "^") {
                    str = AMremoveCharsAndBlanks(str, sym2.input.length);
                    var res2 = AMparseSexpr(str);
                    str = res2[1];
                    tag = res2[2];  // leave space between x_1^2 and a following sin etc.
                    node = AMcreateMmlNode((underover ? "munderover" : "msubsup"), node);
                    node.appendChild(result[0]);
                    node.appendChild(res2[0]);
                } else if (symbol.input == "_") {
                    node = AMcreateMmlNode((underover ? "munder" : "msub"), node);
                    node.appendChild(result[0]);
                } else {
                    node = AMcreateMmlNode((underover ? "mover" : "msup"), node);
                    node.appendChild(result[0]);
                }
                node = AMcreateMmlNode("mrow", node); // so sum does not stretch
            } else {
                node = AMcreateMmlNode(symbol.tag, node);
                if (symbol.input == "\\atop" || symbol.input == "\\choose")
                    node.setAttribute("linethickness", "0ex");
                node.appendChild(result[0]);
                if (symbol.input == "\\choose")
                    node = AMcreateMmlNode("mfenced", node);
            }
        }
        return [node, str, tag];
    }

    function AMparseExpr(str, rightbracket, matrix) {
        var symbol, node, result, i, tag;
        var newFrag = document.createDocumentFragment();
        do {
            str = AMremoveCharsAndBlanks(str, 0);
            result = AMparseIexpr(str);
            node = result[0];
            str = result[1];
            tag = result[2];
            symbol = AMgetSymbol(str);
            if (node != undefined) {
                if ((tag == "mn" || tag == "mi") && symbol != null && typeof symbol.func == "boolean" && symbol.func) {
                    // Add space before \sin in 2\sin x or x\sin x
                    var space = AMcreateElementMathML("mspace");
                    space.setAttribute("width", "0.167em");
                    node = AMcreateMmlNode("mrow", node);
                    node.appendChild(space);
                }
                newFrag.appendChild(node);
            }
        } while ((symbol.ttype != RIGHTBRACKET) && symbol != null && symbol.output != "");
        tag = null;
        if (symbol.ttype == RIGHTBRACKET) {
            if (symbol.input == "\\right") { // right what?
                str = AMremoveCharsAndBlanks(str, symbol.input.length);
                symbol = AMgetSymbol(str);
                if (symbol != null && symbol.input == ".")
                    symbol.invisible = true;
                if (symbol != null)
                    tag = symbol.rtag;
            }
            if (symbol != null)
                str = AMremoveCharsAndBlanks(str, symbol.input.length); // ready to return
            var len = newFrag.childNodes.length;
            if (matrix && len > 0 && newFrag.childNodes[len - 1].nodeName == "mrow" && len > 1 && newFrag.childNodes[len - 2].nodeName == "mo" && newFrag.childNodes[len - 2].firstChild.nodeValue == "&") { //matrix
                var pos = []; // positions of ampersands
                var m = newFrag.childNodes.length;
                for (i = 0; matrix && i < m; i = i + 2) {
                    pos[i] = [];
                    node = newFrag.childNodes[i];
                    for (var j = 0; j < node.childNodes.length; j++)
                        if (node.childNodes[j].firstChild && node.childNodes[j].firstChild.nodeValue == "&")
                            pos[i][pos[i].length] = j;
                }
                var row, frag, n, k, table = document.createDocumentFragment();
                for (i = 0; i < m; i = i + 2) {
                    row = document.createDocumentFragment();
                    frag = document.createDocumentFragment();
                    node = newFrag.firstChild; // <mrow> -&-&...&-&- </mrow>
                    n = node.childNodes.length;
                    k = 0;
                    for (j = 0; j < n; j++) {
                        if (typeof pos[i][k] != "undefined" && j == pos[i][k]) {
                            node.removeChild(node.firstChild); //remove &
                            row.appendChild(AMcreateMmlNode("mtd", frag));
                            k++;
                        } else frag.appendChild(node.firstChild);
                    }
                    row.appendChild(AMcreateMmlNode("mtd", frag));
                    if (newFrag.childNodes.length > 2) {
                        newFrag.removeChild(newFrag.firstChild); //remove <mrow> </mrow>
                        newFrag.removeChild(newFrag.firstChild); //remove <mo>&</mo>
                    }
                    table.appendChild(AMcreateMmlNode("mtr", row));
                }
                return [table, str];
            }
            if (typeof symbol.invisible != "boolean" || !symbol.invisible) {
                node = AMcreateMmlNode("mo", document.createTextNode(symbol.output));
                newFrag.appendChild(node);
            }
        }
        return [newFrag, str, tag];
    }

    //规范化Latx
    function FixLatex(str) {
        str = str.replace(/^\$|\$$/g, ""); //去除2端的$
        str = str.replace(/^\s+/g, "");

        str = str.replace(/（/g, "(");
        str = str.replace(/）/g, ")");

        str = str.replace(/\\\\([\(\)\[\]\{\}])/g, "\\\\ $1");
        str = str.replace(/\{([\(\)\[\]])\}/g, "$1");
        str = str.replace(/\\(left|right)\s*([\[\]\(\)])/g, "$2");
        str = str.replace(/\\(left|right)\s*\\([\{\}])/g, "\\$2");

        str = str.replace(/\\{/g, "\\left\\{");
        str = str.replace(/\\}/g, "\\right\\}");

        str = str.replace(/\(/g, "\\left(");
        str = str.replace(/\)/g, "\\right)");

        str = str.replace(/\[/g, "\\left[");
        str = str.replace(/\]/g, "\\right]");
        return str;
    }

    //解析给定documentElement的核心方法
    function AMtranslateDOM(el, callBack) {
        if (el == null || el.nodeType == 8 || el.childNodes.length < 1 || new RegExp("input|form|math|iframe|textarea|pre|svg").test(el.nodeName) || typeof (callBack) != "function")
            return;

        var str, frg, cl, arr, li;
        cl = el.childNodes.length;
        for (var i = cl - 1; -1 < i; i--) {
            if (el.childNodes[i].nodeType != 3) {
                AMtranslateDOM(el.childNodes[i], callBack);
                continue;
            }
            str = el.childNodes[i].nodeValue;
            if (str == null || str.indexOf("\$") < 0) continue;

            li = 0;
            str = str.replace(/\\\$/g, "￦"); //为了方便替换\$为￦
            frg = document.createDocumentFragment();
            str.replace(/\$[\s\S]*?\$/ig, function (m, index) {
                if (li != index && index > 0) {
                    frg.appendChild(document.createTextNode(str.substr(li, index - li).replace(/￦/g, "\$")));
                }
                frg.appendChild(callBack(m.replace(/￦/g, "\$")));
                li = index + m.length;
                return m;
            });
            if (li != str.length)
                frg.appendChild(document.createTextNode(str.substr(li).replace(/￦/g, "\$")));

            el.replaceChild(frg, el.childNodes[i]);
        }
    }
    /////////////////////////////////////////////////////////私有方法变量/////////////////////////////////////////////


    /////////////////////////////////////////////////////////公有方法变量/////////////////////////////////////////////

    //检测浏览器是否具备了展现公式的条件
    this.CanRender = hasMathPlayer || BrowserIS.isFirefox;

    //显示公式显示软件安装向导
    var noteExist = false; //防止重复加载提示信息的标志变量
    this.ShowNote = function () {
        var info;
        if (noteExist) {
            info = document.getElementById("MRSoftSetupGuide");
            if (info) {
                info.style.display = "";
                return;
            }
        }
        info = AMcreateElementXHTML("div");
        info.id = "MRSoftSetupGuide";
        info.style.cssText = "position: fixed; _position: absolute; z-index: 9999; opacity: 0.9; background-color: #7fb80e;vertical-align: middle; font-size: 14px; padding: 8px; bottom: 0; width: 99%;";
        info.innerHTML = '<a href="javascript:void(0)" onclick="this.parentNode.style.display=\'none\'"><img alt="关闭" style="float: right;" src="http://img.jyeoo.net/www/support/mathplayer/close.png" /></a>' +
        '<div style="background: #F1F5FA;">' +
            '<div style="margin:0 auto;font-size:16px;text-align:center;line-height:50px">' +
                '<span>如果公式符号显示乱码，菁优网建议您下载并安装：<a href="http://www.jyeoo.com/support/MPSetup" target="_blank" style="color:Black;"><u><b>公式显示软件<b></u></a></span>' +
            '</div>' +
        '</div>';
        document.body.appendChild(info);
        noteExist = true;
    };

    this.ToMathML = function (str, mlstyle) {
        //加载转换列表
        if (AMnames.length < 1) { AMinitSymbols(); }
        if (typeof (str) != "string") { return null };

        var temp, result, node, dstr, style;
        dstr = FixLatex(str);
        result = AMcreateElementMathML("mstyle");
        node = AMcreateElementMathML("mstyle");
        style = mlstyle || mstyleAttr;
        for (var item in style) {
            temp = style[item];
            if (typeof (temp) == "string" && temp.replace(/^\s+/g, "") != "") { node.setAttribute(item, temp); }
        }

        node.appendChild(AMparseExpr(dstr, false, false)[0]);
        node = AMcreateMmlNode("math", node);
        node.setAttribute("latex", str.replace(/^\$|\$$/g, "")); //此处一定要去除首末的$否则可能会造成多次解析
        node.setAttribute("title", node.getAttribute("latex"));
        return node;
    };

    //从服务器解析Latex
    this.ServerParse = function (str) {
        return false;
        var dstr = FixLatex(str);
        dstr = dstr.replace(/×/g, "\\times ");
        dstr = dstr.replace(/±/g, "\\pm ");
        dstr = dstr.replace(/[＞>]/ig, "\\gt ");
        dstr = dstr.replace(/[＜<]/ig, "\\lt ");
        dstr = dstr.replace(/÷/ig, "\\div ");
        dstr = dstr.replace(/π/ig, "\\pi ");
        dstr = dstr.replace(/α/ig, "\\alpha ");
        dstr = dstr.replace(/β/ig, "\\beta ");
        dstr = dstr.replace(/γ/ig, "\\gamma ");
        dstr = dstr.replace(/，/ig, ",");
        dstr = dstr.replace(/；/ig, ";");
        dstr = dstr.replace(/：/ig, ":");
        dstr = dstr.replace(/[•．]/ig, "\\cdot ");
        dstr = dstr.replace(/①/g, "(1)");
        dstr = dstr.replace(/②/g, "(2)");
        dstr = dstr.replace(/③/g, "(3)");
        dstr = dstr.replace(/°/g, "^o");

        dstr = dstr.replace(/[⋯…]/g, "...");
        dstr = dstr.replace(/⋱/g, "\\ddots");
        dstr = dstr.replace(/′/g, "'");
        dstr = dstr.replace(/″/g, "''");
        dstr = dstr.replace(/、/g, ",");
        dstr = dstr.replace(/≥/g, "\\ge ");
        dstr = dstr.replace(/≤/g, "\\le ");
        dstr = dstr.replace(/≠/g, "\\ne ");
        dstr = dstr.replace(/≈/g, "\\approx ");
        dstr = dstr.replace(/∵/g, "\\because ");
        dstr = dstr.replace(/∴/g, "\\therefore ");
        dstr = dstr.replace(/∠/g, "\\angle ");

        dstr = dstr.replace(/\\root\s*{([^{]+?)}/ig, "\\sqrt[$1]");
        var cnode = AMcreateElementXHTML("img");
        cnode.setAttribute("latex", str.replace(/^\$|\$$/g, "")); //此处一定要去除首末的$
        cnode.setAttribute("title", "数学公式—如显示乱码，请安装公式显示软件MathPlayer！");
        cnode.setAttribute("alt", "公式图片");
        cnode.setAttribute("src", mathMLApiUri[3] + encodeURIComponent(dstr));
        cnode.style.marginLeft = 0;
        cnode.style.marginRight = 0;
        cnode.style.verticalAlign = "middle";
        cnode.style.border = 0;
        return cnode;
    };

    //解析给定documentElement中的所有公式内容
    //el:Element
    //执行自定义的方法
    //callBack必须返回字符串的形式
    this.ParseDOM = function (el, callBack) {
        AMtranslateDOM(el, callBack);
        if (isIE && hasMathPlayer) { //需要匹配公式的字体大小和周围的文本
            var dmath = document.getElementsByTagName('math');
            for (var i = 0; i < dmath.length; i++) dmath[i].update()
        }
    };

    //默认的解析方法
    this.Render = function (el, mlstyle) {
        if (!self.CanRender) {//未安装mathplay则采用图片展示
            var hasEQ = false;
            //采用mathjye处理公式
            AMtranslateDOM(el, function (str) {
                hasEQ = true;
                return self.ServerParse(str);
            });

            //如果包含公式且是IE则提示安装插件
            if (hasEQ && BrowserIS.isMSIE) { self.ShowNote(); }
        } else {//反之采用mathplay显示公式
            AMtranslateDOM(el, function (str) {
                return self.ToMathML(str, mlstyle);
            });
            if (isIE && hasMathPlayer) { //需要匹配公式的字体大小和周围的文本
                var dmath = document.getElementsByTagName('math');
                for (var i = 0; i < dmath.length; i++) dmath[i].update()
            }
        }
    };

};

//实例化一个全局对象供调用
var LatexRender;
var MathJye;
try { LatexRender = new LatexPageRender(); MathJye = new htmlLatexRender(); } catch (e) { }

//IE中使用mathplayer来显示公式，IE中下面的内容是必须要输出的即便没有安装mathplay也要输出，否则命名空间不正确
if (BrowserIS.isMSIE) {
    if (hasMathPlayer) {
        document.write("<object id=\"mathplayer\" classid=\"clsid:32F66A20-7614-11D4-BD11-00104BD3F987\"></object>");
    }
    document.write("<?import namespace=\"m\" implementation=\"#mathplayer\"?>");
}

function renderLatex(el) {
    try {
        MathJye.LayOut(el);
        LatexRender.Render(el, { color: "black", fontsize: "13px", fontfamily: "arial", displaystyle: "true" });
    } catch (e) { }
}
//公式解析结束



//特殊含义字母正斜体处理
function isItalic() {

}


